package h264_nvenc

import (
	"errors"
	"git.kor-elf.net/kor-elf/gui-for-ffmpeg/internal/ffmpeg/encoder"
)

var Presets = []string{
	"default",
	"slow",
	"medium",
	"fast",
	"hp",
	"hq",
	"bd",
	"ll",
	"llhq",
	"llhp",
	"lossless",
	"losslesshp",
}

func NewEncoder() encoder.EncoderContract {
	parameters := map[string]encoder.ParameterContract{
		"preset": newParameterPreset(),
	}
	getParams := func(parameters map[string]encoder.ParameterContract) []string {
		params := []string{"-c:v", "h264_nvenc"}

		if parameters["preset"] != nil && parameters["preset"].IsEnabled() {
			params = append(params, "-preset", parameters["preset"].Get())
		}

		return params
	}

	return encoder.NewEncoder("h264_nvenc", parameters, getParams)
}

func NewData() encoder.EncoderDataContract {
	title := "h264_nvenc"
	formats := []string{"mp4"}
	fileType := encoder.FileType(encoder.Video)
	return encoder.NewData(title, formats, fileType, NewEncoder)
}

func newParameterPreset() encoder.ParameterContract {
	setParameter := func(s string) (string, error) {
		for _, value := range Presets {
			if value == s {
				return value, nil
			}
		}

		return "", errors.New("preset not found")
	}
	return encoder.NewParameter("preset", false, "default", setParameter)
}
